@@ -43,13 +43,14 @@ from solfege import uuid
 from solfege import xrandom
 from solfege.dataparser import istr
 from solfege.mpd import mpdutils
+from solfege.mpd.duration import Duration
 from solfege.mpd.rat import Rat
 
 _test_mode = False
 
 # The name of the folder containing the standard learning trees.
 # Relative to the installation dir.
-exercises_dir = u"exercises/standard"
+exercises_dir = os.path.join(u"exercises", u"standard")
 _abs_exercises_dir = os.path.join(os.getcwdu(), exercises_dir) + os.sep
 
 solfege_uri = u"solfege:"
@@ -161,6 +162,9 @@ keywords = (
     'rhythmtapping',
     'rhythmtapping2',
     'idproperty',
+    'rhythmdictation',
+    'rhythmdictation2',
+    'tuner',
     #
     'harmonic',
     'melodic',
@@ -373,6 +377,111 @@ class _Header(dict):
             return self[name]
         return istr("")
 
+class Bar(list):
+    class Item(object):
+        """
+        An object representing either a note or a rest.
+        """
+        def __init__(self, isnote, duration, tie):
+            assert isinstance(duration, Duration)
+            self.m_isnote = isnote
+            self.m_duration = duration
+            self.m_tie = tie
+    class BadDuration(Exception):
+        pass
+    class FullException(Exception):
+        pass
+    stepwidth = 30
+    def __init__(self, keysig):
+        assert isinstance(keysig, Rat)
+        self.m_keysig = keysig
+    def _do_add(self, isnote, n):
+        if not isinstance(n, Duration):
+            if not isinstance(n, unicode):
+                raise self.BadDuration(n)
+            n = Duration.new_from_string(n)
+        if self.duration() + n.get_rat_value() > self.m_keysig:
+            raise self.FullException()
+        self.append(self.Item(isnote, n, False))
+    def isfull(self):
+        return self.duration() == self.m_keysig
+    def can_append(self, item):
+        """
+        Return True if the bar is not full and there is space to add item.
+        """
+        return (self.duration()
+                + item.m_duration.get_rat_value() <= self.m_keysig)
+    def add_note(self, n):
+        self._do_add(True, n)
+    def add_rest(self, n):
+        self._do_add(False, n)
+    def duration(self):
+        r = Rat(0)
+        for item in self:
+            r += item.m_duration.get_rat_value()
+        return r
+    def do_layout(self, show_timesig):
+        self.m_show_timesig = show_timesig
+        if show_timesig:
+            xpos = self.stepwidth
+        else:
+            xpos = self.stepwidth / 2
+        for i in self:
+            i.m_xpos = xpos
+            xpos += 30
+        #If the bar is full, then the width of the bar is the m_xpos
+        #of the last note + Bar.stepwidth
+        if self.isfull():
+            self.m_width = self[-1].m_xpos + Bar.stepwidth
+        self.m_width = xpos + int((self.m_keysig - self.duration()) * 4 * Bar.stepwidth)
+    def _get_timelist(self):
+        """
+        A timelist is a list of list:
+        [isnote, duration] of the types [bool, Rat]
+        """
+        v = []
+        tie = False
+        for item in self:
+            if tie:
+                v[-1][1] += item.m_duration.get_rat_value()
+            else:
+                v.append([item.m_isnote, item.m_duration.get_rat_value()])
+            tie = item.m_tie
+        return v
+
+
+class RhythmStaff(list):
+    """
+    A staff is a list of Bars
+    A object that know how long rhythm to enter, and will
+    know of all time signature changes and bar lines.
+
+    A question in the lesson file could be written like this:
+    \time 3/4 c4 c8 c8 c4 | \time 3/8 c8 c8 c16 c16
+    or:
+    4 8 8 4 | 8 8 16 16
+
+    We need code that can transform the former to the last format so
+    that we can compare the real music to the rhythm entered by
+    the user.
+    """
+    def do_layout(self):
+        timesig = None
+        xpos = 0
+        for bar in self:
+            bar.m_xpos = xpos
+            bar.do_layout(timesig != bar.m_keysig)
+            timesig = bar.m_keysig
+            xpos += bar.m_width
+    def get_timelist(self):
+        v = []
+        for bar in self:
+            v.extend(bar._get_timelist())
+        return v
+    def append(self, bar):
+        if not isinstance(bar, Bar):
+            raise TypeError("RhythmStaff.append: only Bar allowed")
+        list.append(self, bar)
 
 class MusicBaseClass(object):
     """
@@ -464,6 +573,40 @@ class MpdParsable(MusicBaseClass):
             # added music code, not in the m_musicdata code.
             return
         exception.m_obj_lineno = exception.m_lineno - 1
+    def get_rhythmstaff(self, lessonfile_ref):
+        """
+        Create and return a RhythmStaff showing the rhythm of the music
+        object.
+        """
+        staff = RhythmStaff()
+        timesig = None
+        bar_content = Rat(0, 1)
+        # the current note/rest-duration
+        duration = Duration(4, 0)
+        for toc_type, toc in mpd.parser.Lexer(
+                self.get_mpd_music_string(lessonfile_ref)):
+            if toc_type in (mpd.parser.Lexer.NOTE, mpd.parser.Lexer.REST):
+                if timesig is None:
+                    # very first note and no timesig parsed
+                    staff.append(Bar(Rat(4, 4)))
+                    timesig = Rat(4, 4)
+                if staff[-1].isfull():
+                    staff.append(Bar(timesig))
+                if toc.m_duration:
+                    duration = toc.m_duration
+                staff[-1].append(Bar.Item(toc_type == mpd.parser.Lexer.NOTE,
+                                duration, False))
+            elif toc_type == mpd.parser.Lexer.TIME:
+                timesig = Rat(toc.m_num, toc.m_den)
+                staff.append(Bar(timesig))
+            elif toc_type == u'~':
+                staff[-1][-1].m_tie = True
+            elif toc_type == u'|':
+                if staff[-1].m_keysig != bar_content:
+                    print "FIXME: barlines not ok."
+        return staff
+            
+ 
 
 class MpdDisplayable(MpdParsable):
     pass
@@ -1621,6 +1764,12 @@ class ElembuilderLessonfile(QuestionsLessonfile):
         super(ElembuilderLessonfile, self).parse_string(s, really_filename)
         # We need the name variable for statistics
         self.discard_questions_without_name()
+        # question['elements'] should always be a list. If a question allows
+        # a single element as answer one gets a dict from parse_string and not
+        # a list. Thus we pack the dict in a list and are happy.
+        for question in self.m_questions:
+            if type(question['elements']) <> (type([])):
+                question['elements'] = [question['elements']]
         # This loop let us use normal strings and lesson file functions
         # like pangomarkup as element labels.
         for question in self.m_questions:
@@ -1682,19 +1831,22 @@ class InfoCache(object):
             self.parse_file(filename, mtime)
         return self._data[filename][field]
     def parse_file(self, filename, mtime=None):
-        logging.debug("parse_file %s" % filename)
         assert is_uri(filename) or os.path.isabs(filename)
         p = parse_lesson_file_header(uri_expand(filename))
         if not p:
             raise self.FileNotLessonfile(filename)
-        self._data[filename] = {
-            'title': p.header.get('title', 'error: no title in file'),
-            'module': p.header['module'],
-            'test': p.header.get('test', None),
-            'test_requirement': p.header.get('test_requirement', None),
-            'mtime': mtime if mtime else os.path.getmtime(uri_expand(filename)),
-            'replaces': p.header.get('replaces', []),
+        try:
+            self._data[filename] = {
+                'title': p.header.get('title', 'error: no title in file'),
+                'module': p.header['module'],
+                'test': p.header.get('test', None),
+                'test_requirement': p.header.get('test_requirement', None),
+                'mtime': mtime if mtime else os.path.getmtime(uri_expand(filename)),
+                'replaces': p.header.get('replaces', []),
         }
+        except KeyError:
+            logging.debug("InfoCache.parse_file: FileNotLessonfile(%s)" % filename)
+            raise self.FileNotLessonfile(filename)
     def iter_parse_all_files(self):
         """
         Parse the files and put then in the cache.
